import { WalletContextState } from "@solana/wallet-adapter-react";
import { Connection } from "@solana/web3.js";
import { MintData, Trans, initializerTokenResult, setupCreateToken, signMessageResult } from "../types/dataTypes";
/**
 * This hook returns:
 * - Initialize New Token Function
 * - Mint Token Function
 *
 * @param {WalletContextState | null} wallet: Use hook *useWallet* from '@solana/wallet-adapter-react' to retrieve
 *
 * @param {Connection} connection: Use hook *useConnection* from '@solana/wallet-adapter-react' to retrieve
 *
 * @param {string} locale: Can be "en" | "vi". Default is "en"
 *
 * @example
 *  const wallet = useWallet();
    const { connection } = useConnection();
    const { initializerToken, mintToken } = useEasyToken(
      wallet,
      connection,
      "en",
    );
 */
export declare const useEasyToken: (wallet: WalletContextState | null, connection: Connection, locale?: Trans) => {
    initializerToken: ({ data, isMainnet, }: setupCreateToken) => Promise<initializerTokenResult>;
    mintToken: ({ mintAccountAddress, amount, decimals, }: MintData) => Promise<{
        txSucceeded: boolean;
        error: any;
        tx?: undefined;
    } | {
        txSucceeded: boolean;
        tx: string;
        error?: undefined;
    }>;
    signMessage: (message: string) => Promise<signMessageResult>;
};
