"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkIfTransactionSucceeded = exports.createAssociatedTokenAccountInstruction = exports.generateAssociatedTokenAccountAddress = void 0;
var tslib_1 = require("tslib");
var spl_token_1 = require("@solana/spl-token");
var web3_js_1 = require("@solana/web3.js");
var MAX_BLOCKHASH_SLOT_DIFF = 150;
var generateAssociatedTokenAccountAddress = function (mint, buyer) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, web3_js_1.PublicKey.findProgramAddressSync([buyer.toBuffer(), spl_token_1.TOKEN_PROGRAM_ID.toBuffer(), mint.toBuffer()], spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID)[0]];
            case 1: return [2 /*return*/, _a.sent()];
        }
    });
}); };
exports.generateAssociatedTokenAccountAddress = generateAssociatedTokenAccountAddress;
var createAssociatedTokenAccountInstruction = function (associatedTokenAddress, payer, walletAddress, splTokenMintAddress) {
    var keys = [
        {
            pubkey: payer,
            isSigner: true,
            isWritable: true,
        },
        {
            pubkey: associatedTokenAddress,
            isSigner: false,
            isWritable: true,
        },
        {
            pubkey: walletAddress,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: splTokenMintAddress,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: web3_js_1.SystemProgram.programId,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: spl_token_1.TOKEN_PROGRAM_ID,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: web3_js_1.SYSVAR_RENT_PUBKEY,
            isSigner: false,
            isWritable: false,
        },
    ];
    return new web3_js_1.TransactionInstruction({
        keys: keys,
        programId: spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID,
        data: Buffer.from([]),
    });
};
exports.createAssociatedTokenAccountInstruction = createAssociatedTokenAccountInstruction;
var checkIfTransactionSucceeded = function (connection, txSignature, lastValidHeight, startTime) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var hashExpired, txSucceeded, status_1, endTime, elapsed, endTime, elapsed;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                hashExpired = false;
                txSucceeded = false;
                _a.label = 1;
            case 1:
                if (!(!hashExpired && !txSucceeded && txSignature)) return [3 /*break*/, 5];
                return [4 /*yield*/, connection.getSignatureStatus(txSignature)];
            case 2:
                status_1 = (_a.sent()).value;
                // Break loop if transaction has succeeded
                if (status_1 &&
                    (status_1.confirmationStatus === "confirmed" ||
                        status_1.confirmationStatus === "finalized")) {
                    txSucceeded = true;
                    endTime = new Date();
                    elapsed = (endTime.getTime() - startTime.getTime()) / 1000;
                    console.log("Transaction Success. Elapsed time: ".concat(elapsed, " seconds."));
                    return [3 /*break*/, 5];
                }
                return [4 /*yield*/, isBlockhashExpired(connection, lastValidHeight)];
            case 3:
                hashExpired = _a.sent();
                // Break loop if blockhash has expired
                if (hashExpired) {
                    endTime = new Date();
                    elapsed = (endTime.getTime() - startTime.getTime()) / 1000;
                    console.log("Blockhash has expired. Elapsed time: ".concat(elapsed, " seconds."));
                    return [3 /*break*/, 5];
                }
                // Check again after 2.5 sec
                return [4 /*yield*/, sleep(2500)];
            case 4:
                // Check again after 2.5 sec
                _a.sent();
                return [3 /*break*/, 1];
            case 5: return [2 /*return*/, txSucceeded];
        }
    });
}); };
exports.checkIfTransactionSucceeded = checkIfTransactionSucceeded;
var sleep = function (ms) {
    return new Promise(function (resolve) { return setTimeout(resolve, ms); });
};
var isBlockhashExpired = function (connection, lastValidBlockHeight) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var currentBlockHeight;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, connection.getBlockHeight("finalized")];
            case 1:
                currentBlockHeight = _a.sent();
                return [2 /*return*/, currentBlockHeight > lastValidBlockHeight - MAX_BLOCKHASH_SLOT_DIFF];
        }
    });
}); };
//# sourceMappingURL=helpers.js.map