import { __awaiter, __generator } from "tslib";
import { ASSOCIATED_TOKEN_PROGRAM_ID, TOKEN_PROGRAM_ID, } from "@solana/spl-token";
import { PublicKey, SYSVAR_RENT_PUBKEY, SystemProgram, TransactionInstruction, } from "@solana/web3.js";
var MAX_BLOCKHASH_SLOT_DIFF = 150;
export var generateAssociatedTokenAccountAddress = function (mint, buyer) { return __awaiter(void 0, void 0, void 0, function () {
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, PublicKey.findProgramAddressSync([buyer.toBuffer(), TOKEN_PROGRAM_ID.toBuffer(), mint.toBuffer()], ASSOCIATED_TOKEN_PROGRAM_ID)[0]];
            case 1: return [2 /*return*/, _a.sent()];
        }
    });
}); };
export var createAssociatedTokenAccountInstruction = function (associatedTokenAddress, payer, walletAddress, splTokenMintAddress) {
    var keys = [
        {
            pubkey: payer,
            isSigner: true,
            isWritable: true,
        },
        {
            pubkey: associatedTokenAddress,
            isSigner: false,
            isWritable: true,
        },
        {
            pubkey: walletAddress,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: splTokenMintAddress,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: SystemProgram.programId,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: TOKEN_PROGRAM_ID,
            isSigner: false,
            isWritable: false,
        },
        {
            pubkey: SYSVAR_RENT_PUBKEY,
            isSigner: false,
            isWritable: false,
        },
    ];
    return new TransactionInstruction({
        keys: keys,
        programId: ASSOCIATED_TOKEN_PROGRAM_ID,
        data: Buffer.from([]),
    });
};
export var checkIfTransactionSucceeded = function (connection, txSignature, lastValidHeight, startTime) { return __awaiter(void 0, void 0, void 0, function () {
    var hashExpired, txSucceeded, status_1, endTime, elapsed, endTime, elapsed;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                hashExpired = false;
                txSucceeded = false;
                _a.label = 1;
            case 1:
                if (!(!hashExpired && !txSucceeded && txSignature)) return [3 /*break*/, 5];
                return [4 /*yield*/, connection.getSignatureStatus(txSignature)];
            case 2:
                status_1 = (_a.sent()).value;
                // Break loop if transaction has succeeded
                if (status_1 &&
                    (status_1.confirmationStatus === "confirmed" ||
                        status_1.confirmationStatus === "finalized")) {
                    txSucceeded = true;
                    endTime = new Date();
                    elapsed = (endTime.getTime() - startTime.getTime()) / 1000;
                    console.log("Transaction Success. Elapsed time: ".concat(elapsed, " seconds."));
                    return [3 /*break*/, 5];
                }
                return [4 /*yield*/, isBlockhashExpired(connection, lastValidHeight)];
            case 3:
                hashExpired = _a.sent();
                // Break loop if blockhash has expired
                if (hashExpired) {
                    endTime = new Date();
                    elapsed = (endTime.getTime() - startTime.getTime()) / 1000;
                    console.log("Blockhash has expired. Elapsed time: ".concat(elapsed, " seconds."));
                    return [3 /*break*/, 5];
                }
                // Check again after 2.5 sec
                return [4 /*yield*/, sleep(2500)];
            case 4:
                // Check again after 2.5 sec
                _a.sent();
                return [3 /*break*/, 1];
            case 5: return [2 /*return*/, txSucceeded];
        }
    });
}); };
var sleep = function (ms) {
    return new Promise(function (resolve) { return setTimeout(resolve, ms); });
};
var isBlockhashExpired = function (connection, lastValidBlockHeight) { return __awaiter(void 0, void 0, void 0, function () {
    var currentBlockHeight;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, connection.getBlockHeight("finalized")];
            case 1:
                currentBlockHeight = _a.sent();
                return [2 /*return*/, currentBlockHeight > lastValidBlockHeight - MAX_BLOCKHASH_SLOT_DIFF];
        }
    });
}); };
//# sourceMappingURL=helpers.js.map